"""A web3.py Contract class for the IIrm contract.

DO NOT EDIT.  This file was generated by pypechain.  See documentation at
https://github.com/delvtech/pypechain"""

# contracts have PascalCase names
# pylint: disable=invalid-name

# contracts control how many attributes and arguments we have in generated code
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-arguments

# we don't need else statement if the other conditionals all have return,
# but it's easier to generate
# pylint: disable=no-else-return

# This file is bound to get very long depending on contract sizes.
# pylint: disable=too-many-lines

# methods are overriden with specific arguments instead of generic *args, **kwargs
# pylint: disable=arguments-differ

# consumers have too many opinions on line length
# pylint: disable=line-too-long


from __future__ import annotations

from typing import Any, Type, cast

from eth_account.signers.local import LocalAccount
from eth_typing import ChecksumAddress, HexStr
from typing_extensions import Self
from web3 import Web3
from web3.contract.contract import (
    Contract,
    ContractConstructor,
    ContractFunction,
    ContractFunctions,
)
from web3.exceptions import FallbackNotFound
from web3.types import ABI, BlockIdentifier, CallOverride, TxParams

from .MarketParamsTypes import MarketParams
from .MarketTypes import Market
from .utilities import (
    dataclass_to_tuple,
    rename_returned_types,
    try_bytecode_hexbytes,
)

structs = {
    "MarketParams": MarketParams,
    "Market": Market,
}


class IIrmBorrowRateContractFunction(ContractFunction):
    """ContractFunction for the borrowRate method."""

    def __call__(self, marketParams: MarketParams, market: Market) -> IIrmBorrowRateContractFunction:  # type: ignore
        clone = super().__call__(
            dataclass_to_tuple(marketParams), dataclass_to_tuple(market)
        )
        self.kwargs = clone.kwargs
        self.args = clone.args
        return self

    def call(
        self,
        transaction: TxParams | None = None,
        block_identifier: BlockIdentifier = "latest",
        state_override: CallOverride | None = None,
        ccip_read_enabled: bool | None = None,
    ) -> int:
        """returns int."""
        # Define the expected return types from the smart contract call

        return_types = int

        # Call the function

        raw_values = super().call(
            transaction, block_identifier, state_override, ccip_read_enabled
        )
        return cast(
            int, rename_returned_types(structs, return_types, raw_values)
        )


class IIrmBorrowRateViewContractFunction(ContractFunction):
    """ContractFunction for the borrowRateView method."""

    def __call__(self, marketParams: MarketParams, market: Market) -> IIrmBorrowRateViewContractFunction:  # type: ignore
        clone = super().__call__(
            dataclass_to_tuple(marketParams), dataclass_to_tuple(market)
        )
        self.kwargs = clone.kwargs
        self.args = clone.args
        return self

    def call(
        self,
        transaction: TxParams | None = None,
        block_identifier: BlockIdentifier = "latest",
        state_override: CallOverride | None = None,
        ccip_read_enabled: bool | None = None,
    ) -> int:
        """returns int."""
        # Define the expected return types from the smart contract call

        return_types = int

        # Call the function

        raw_values = super().call(
            transaction, block_identifier, state_override, ccip_read_enabled
        )
        return cast(
            int, rename_returned_types(structs, return_types, raw_values)
        )


class IIrmContractFunctions(ContractFunctions):
    """ContractFunctions for the IIrm contract."""

    borrowRate: IIrmBorrowRateContractFunction

    borrowRateView: IIrmBorrowRateViewContractFunction

    def __init__(
        self,
        abi: ABI,
        w3: "Web3",
        address: ChecksumAddress | None = None,
        decode_tuples: bool | None = False,
    ) -> None:
        super().__init__(abi, w3, address, decode_tuples)
        self.borrowRate = IIrmBorrowRateContractFunction.factory(
            "borrowRate",
            w3=w3,
            contract_abi=abi,
            address=address,
            decode_tuples=decode_tuples,
            function_identifier="borrowRate",
        )
        self.borrowRateView = IIrmBorrowRateViewContractFunction.factory(
            "borrowRateView",
            w3=w3,
            contract_abi=abi,
            address=address,
            decode_tuples=decode_tuples,
            function_identifier="borrowRateView",
        )


iirm_abi: ABI = cast(
    ABI,
    [
        {
            "type": "function",
            "name": "borrowRate",
            "inputs": [
                {
                    "name": "marketParams",
                    "type": "tuple",
                    "internalType": "struct MarketParams",
                    "components": [
                        {
                            "name": "loanToken",
                            "type": "address",
                            "internalType": "address",
                        },
                        {
                            "name": "collateralToken",
                            "type": "address",
                            "internalType": "address",
                        },
                        {
                            "name": "oracle",
                            "type": "address",
                            "internalType": "address",
                        },
                        {
                            "name": "irm",
                            "type": "address",
                            "internalType": "address",
                        },
                        {
                            "name": "lltv",
                            "type": "uint256",
                            "internalType": "uint256",
                        },
                    ],
                },
                {
                    "name": "market",
                    "type": "tuple",
                    "internalType": "struct Market",
                    "components": [
                        {
                            "name": "totalSupplyAssets",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "totalSupplyShares",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "totalBorrowAssets",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "totalBorrowShares",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "lastUpdate",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "fee",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                    ],
                },
            ],
            "outputs": [
                {"name": "", "type": "uint256", "internalType": "uint256"}
            ],
            "stateMutability": "nonpayable",
        },
        {
            "type": "function",
            "name": "borrowRateView",
            "inputs": [
                {
                    "name": "marketParams",
                    "type": "tuple",
                    "internalType": "struct MarketParams",
                    "components": [
                        {
                            "name": "loanToken",
                            "type": "address",
                            "internalType": "address",
                        },
                        {
                            "name": "collateralToken",
                            "type": "address",
                            "internalType": "address",
                        },
                        {
                            "name": "oracle",
                            "type": "address",
                            "internalType": "address",
                        },
                        {
                            "name": "irm",
                            "type": "address",
                            "internalType": "address",
                        },
                        {
                            "name": "lltv",
                            "type": "uint256",
                            "internalType": "uint256",
                        },
                    ],
                },
                {
                    "name": "market",
                    "type": "tuple",
                    "internalType": "struct Market",
                    "components": [
                        {
                            "name": "totalSupplyAssets",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "totalSupplyShares",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "totalBorrowAssets",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "totalBorrowShares",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "lastUpdate",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                        {
                            "name": "fee",
                            "type": "uint128",
                            "internalType": "uint128",
                        },
                    ],
                },
            ],
            "outputs": [
                {"name": "", "type": "uint256", "internalType": "uint256"}
            ],
            "stateMutability": "view",
        },
    ],
)
# pylint: disable=line-too-long
iirm_bytecode = HexStr("0x")


class IIrmContract(Contract):
    """A web3.py Contract class for the IIrm contract."""

    abi: ABI = iirm_abi
    bytecode: bytes | None = try_bytecode_hexbytes(iirm_bytecode, "iirm")

    def __init__(self, address: ChecksumAddress | None = None) -> None:
        try:
            # Initialize parent Contract class
            super().__init__(address=address)
            self.functions = IIrmContractFunctions(iirm_abi, self.w3, address)  # type: ignore

        except FallbackNotFound:
            print("Fallback function not found. Continuing...")

    functions: IIrmContractFunctions

    @classmethod
    def constructor(cls) -> ContractConstructor:  # type: ignore
        """Creates a transaction with the contract's constructor function.

        Parameters
        ----------

        w3 : Web3
            A web3 instance.
        account : LocalAccount
            The account to use to deploy the contract.

        Returns
        -------
        Self
            A deployed instance of the contract.

        """

        return super().constructor()

    @classmethod
    def deploy(cls, w3: Web3, account: LocalAccount | ChecksumAddress) -> Self:
        """Deploys and instance of the contract.

        Parameters
        ----------
        w3 : Web3
            A web3 instance.
        account : LocalAccount
            The account to use to deploy the contract.

        Returns
        -------
        Self
            A deployed instance of the contract.
        """
        deployer = cls.factory(w3=w3)
        constructor_fn = deployer.constructor()

        # if an address is supplied, try to use a web3 default account
        if isinstance(account, str):
            tx_hash = constructor_fn.transact({"from": account})
            tx_receipt = w3.eth.wait_for_transaction_receipt(tx_hash)

            deployed_contract = deployer(address=tx_receipt.contractAddress)  # type: ignore
            return deployed_contract

        # otherwise use the account provided.
        deployment_tx = constructor_fn.build_transaction()
        current_nonce = w3.eth.get_transaction_count(account.address, "pending")
        deployment_tx.update({"nonce": current_nonce})

        # Sign the transaction with local account private key
        signed_tx = account.sign_transaction(deployment_tx)

        # Send the signed transaction and wait for receipt
        tx_hash = w3.eth.send_raw_transaction(signed_tx.rawTransaction)
        tx_receipt = w3.eth.wait_for_transaction_receipt(tx_hash)

        deployed_contract = deployer(address=tx_receipt.contractAddress)  # type: ignore
        return deployed_contract

    @classmethod
    def factory(
        cls, w3: Web3, class_name: str | None = None, **kwargs: Any
    ) -> Type[Self]:
        """Deploys and instance of the contract.

        Parameters
        ----------
        w3 : Web3
            A web3 instance.
        class_name: str | None
            The instance class name.

        Returns
        -------
        Self
            A deployed instance of the contract.
        """
        contract = super().factory(w3, class_name, **kwargs)
        contract.functions = IIrmContractFunctions(iirm_abi, w3, None)

        return contract
